
/* ###--------------------------------------------------------------### */
/*									*/
/* file		: vh_asimut.h						*/
/* date		: Oct 26 1993						*/
/* version	: v1.93							*/
/* author	: VUONG H.N.						*/
/* content	: declaration of functions and global variables used by	*/
/*		  the simulator						*/
/*									*/
/* ###--------------------------------------------------------------### */

extern char *getenv();

extern int	vhu_error();
extern char    *vhu_avers();

extern struct dct_entry **vhu_initab();
extern void               vhu_fretab();

struct lofig   *VHS_LOFPNT = NULL;      /* current LOFIG pointer        */

struct lkdfig *VHL_HEDLKF=NULL;

extern int                vhl_linker ();
extern int                vhl_restor ();
extern lofig_list        *vhl_makgst ();
extern void               vhx_redpat ();
extern void               vhx_savall ();
extern char               vhx_initfig ();
extern void               vhx_inireg ();
extern void               vhx_core ();
extern void               beh_freabl ();

/* ###--------------------------------------------------------------### */

struct chain *VHX_EVENT  = NULL;
struct chain *VHX_SIGUPD = NULL;
struct chain *VHX_BUSUPD = NULL;
struct chain *VHX_BUXUPD = NULL;
struct chain *VHX_REGUPD = NULL;
struct chain *VHX_AUXUPD = NULL;
struct chain *VHX_BUSERR = NULL;
struct chain *VHX_BUXERR = NULL;
struct chain *VHX_REGERR = NULL;
struct chain *VHX_MSGERR = NULL;
